from typing import Optional, List, Tuple, Any
from argparse import Namespace
from PIL import Image
from PIL.ImageTk import PhotoImage


class Model():
    def __init__(self, args):
        self.args: Namespace = args
        self.images: List[str] = []
        self.selection_box: Optional[Any] = None
        self.press_coord: Tuple[int, int] = (0, 0)
        self.move_coord: Tuple[int, int] = (0, 0)
        self.displayed_image: Optional[PhotoImage] = None
        self.canvas_image: Optional[Any] = None
        self.canvas_image_dimensions: Tuple[int, int] = (0, 0)
        self.current_image: Optional[Image] = None
        self.enabled_selection_mode: bool = False
        self.box_selected: bool = False
        self.current_file: int = 0



import itertools
import mimetypes
import os

from typing import Optional, List, Tuple

from PIL import Image, ImageTk

from inbac.model import Model
from inbac.view import View


class Controller():
    def __init__(self, model: Model, view: View):
        self.model: Model = model
        self.view: View = view

        self.load_images()

    def select_images_folder(self):
        input_dir = self.view.ask_directory()
        if input_dir:
            self.model.args.input_dir = input_dir
            self.model.args.output_dir = os.path.join(
                self.model.args.input_dir, "crops")

            if not os.path.exists(self.model.args.output_dir):
                self.create_output_directory()

    def create_output_directory(self):
        try:
            os.makedirs(self.model.args.output_dir)
        except OSError:
            self.view.show_error(
                "Error", "Output directory cannot be created, please select output directory location")
            self.model.args.output_dir = self.view.ask_directory()

    def load_image(self, image_name: str):
        if self.model.current_image is not None:
            self.model.current_image.close()
            self.model.current_image = None
        image = Image.open(os.path.join(self.model.args.input_dir, image_name))
        self.display_image_on_canvas(image)
        self.view.set_title(image_name)

    def load_images(self):
        if self.model.args.input_dir:
            try:
                self.model.images = self.load_image_list(
                    self.model.args.input_dir)
            except OSError:
                self.view.show_error(
                    "Error", "Input directory cannot be opened")

        if self.model.images:
            try:
                self.model.current_file = 0
                self.load_image(self.model.images[self.model.current_file])
            except IOError:
                self.next_image()

    def display_image_on_canvas(self, image: Image):
        self.clear_canvas()
        self.model.current_image = image
        self.model.canvas_image_dimensions = self.calculate_canvas_image_dimensions(self.model.current_image.size[0],
                                                                                    self.model.current_image.size[1],
                                                                                    self.view.image_canvas.winfo_width(),
                                                                                    self.view.image_canvas.winfo_height())
        displayed_image: Image = self.model.current_image.copy()
        displayed_image.thumbnail(
            self.model.canvas_image_dimensions, Image.ANTIALIAS)
        self.model.displayed_image = ImageTk.PhotoImage(displayed_image)
        self.model.canvas_image = self.view.display_image(
            self.model.displayed_image)

    def clear_canvas(self):
        self.clear_selection_box()
        if self.model.canvas_image is not None:
            self.view.remove_from_canvas(self.model.canvas_image)
            self.model.canvas_image = None

    def clear_selection_box(self):
        if self.model.selection_box is not None:
            self.view.remove_from_canvas(self.model.selection_box)
            self.model.selection_box = None

    def update_selection_box(self):
        selected_box: Tuple[int, int, int, int] = self.get_selected_box(
            self.model.press_coord, self.model.move_coord, self.model.args.aspect_ratio)

        if self.model.selection_box is None:
            self.model.selection_box = self.view.create_rectangle(
                selected_box, self.model.args.selection_box_color)
        else:
            self.view.change_canvas_object_coords(
                self.model.selection_box, selected_box)

    def stop_selection(self):
        self.model.box_selected = False

    def start_selection(self, press_coord: Tuple[int, int]):
        self.model.press_coord = press_coord
        self.model.move_coord = press_coord
        if self.model.enabled_selection_mode and self.model.selection_box is not None:
            selected_box: Tuple[int, int, int, int] = self.view.get_canvas_object_coords(
                self.model.selection_box)
            self.model.box_selected = self.coordinates_in_selection_box(
                self.model.press_coord, selected_box)
        else:
            self.clear_selection_box()

    def move_selection(self, move_coord: Tuple[int, int]):
        if self.model.enabled_selection_mode and not self.model.box_selected:
            return
        prev_move_coord: Tuple[int, int] = self.model.move_coord
        self.model.move_coord = move_coord
        if self.model.box_selected:
            x_delta: int = self.model.move_coord[0] - prev_move_coord[0]
            y_delta: int = self.model.move_coord[1] - prev_move_coord[1]
            self.view.move_canvas_object_by_offset(
                self.model.selection_box, x_delta, y_delta)
        else:
            self.update_selection_box()

    def next_image(self):
        if self.model.current_file + 1 >= len(self.model.images):
            return
        self.model.current_file += 1
        try:
            self.load_image(self.model.images[self.model.current_file])
        except IOError:
            self.next_image()

    def previous_image(self):
        if self.model.current_file - 1 < 0:
            return
        self.model.current_file -= 1
        try:
            self.load_image(self.model.images[self.model.current_file])
        except IOError:
            self.previous_image()

    def save_next(self):
        if self.save():
            self.next_image()

    def save(self) -> bool:
        if self.model.selection_box is None:
            return False
        selected_box: Tuple[int, int, int, int] = self.view.get_canvas_object_coords(
            self.model.selection_box)
        box: Tuple[int, int, int, int] = self.get_real_box(
            selected_box, self.model.current_image.size, self.model.canvas_image_dimensions)
        new_filename: str = self.find_available_name(
            self.model.args.output_dir, self.model.images[self.model.current_file])
        saved_image: Image = self.model.current_image.copy().crop(box)
        if self.model.args.resize:
            saved_image = saved_image.resize(
                (self.model.args.resize[0], self.model.args.resize[1]), Image.LANCZOS)
        if self.model.args.image_format:
            new_filename, _ = os.path.splitext(new_filename)
        saved_image.save(os.path.join(self.model.args.output_dir, new_filename),
                         self.model.args.image_format, quality=self.model.args.image_quality)
        self.clear_selection_box()
        return True

    @staticmethod
    def calculate_canvas_image_dimensions(image_width: int,
                                          image_height: int,
                                          canvas_width: int,
                                          canvas_height: int) -> Tuple[int, int]:
        if image_width > canvas_width or image_height > canvas_height:
            width_ratio: float = canvas_width / image_width
            height_ratio: float = canvas_height / image_height
            ratio: float = min(width_ratio, height_ratio)
            new_image_width: int = int(image_width * ratio)
            new_image_height: int = int(image_height * ratio)
            return (new_image_width, new_image_height)
        return (image_width, image_height)

    @staticmethod
    def load_image_list(directory: str) -> List[str]:
        images: List[str] = []

        for filename in os.listdir(directory):
            filetype, _ = mimetypes.guess_type(filename)
            if filetype is None or filetype.split("/")[0] != "image":
                continue
            images.append(filename)

        return images

    @staticmethod
    def coordinates_in_selection_box(coordinates: Tuple[int, int], selection_box: Tuple[int, int]) -> bool:
        return (coordinates[0] > selection_box[0] and coordinates[0] < selection_box[2]
                and coordinates[1] > selection_box[1] and coordinates[1] < selection_box[3])

    @staticmethod
    def find_available_name(directory: str, filename: str) -> str:
        name, extension = os.path.splitext(filename)
        if not os.path.isfile(os.path.join(directory, filename)):
            return filename
        for num in itertools.count(2):
            if not os.path.isfile(os.path.join(directory, name + str(num) + extension)):
                return name + str(num) + extension

    @staticmethod
    def get_selection_box_for_aspect_ratio(selection_box: Tuple[int, int, int, int],
                                           aspect_ratio: float,
                                           mouse_press_coord: Tuple[int, int],
                                           mouse_move_coord: Tuple[int, int]) -> Tuple[int, int, int, int]:
        selection_box: List[int] = list(selection_box)
        width: int = selection_box[2] - selection_box[0]
        height: int = selection_box[3] - selection_box[1]
        if float(width)/float(height) > aspect_ratio:
            height = round(width / aspect_ratio)
            if mouse_move_coord[1] > mouse_press_coord[1]:
                selection_box[3] = selection_box[1] + height
            else:
                selection_box[1] = selection_box[3] - height
        else:
            width = round(height * aspect_ratio)
            if mouse_move_coord[0] > mouse_press_coord[0]:
                selection_box[2] = selection_box[0] + width
            else:
                selection_box[0] = selection_box[2] - width
        return tuple(selection_box)

    @staticmethod
    def get_selected_box(mouse_press_coord: Tuple[int, int], mouse_move_coord: Tuple[int, int],
                         aspect_ratio: Optional[Tuple[int, int]]) -> Tuple[int, int, int, int]:
        selection_top_left_x: int = min(
            mouse_press_coord[0], mouse_move_coord[0])
        selection_top_left_y: int = min(
            mouse_press_coord[1], mouse_move_coord[1])
        selection_bottom_right_x: int = max(
            mouse_press_coord[0], mouse_move_coord[0])
        selection_bottom_right_y: int = max(
            mouse_press_coord[1], mouse_move_coord[1])
        selection_box: Tuple[int, int, int, int] = (selection_top_left_x, selection_top_left_y,
                                                    selection_bottom_right_x, selection_bottom_right_y)

        if aspect_ratio is not None:
            aspect_ratio: float = float(aspect_ratio[0])/float(aspect_ratio[1])
            try:
                selection_box: Tuple[int, int, int, int] = Controller.get_selection_box_for_aspect_ratio(selection_box, aspect_ratio,
                                                                                                         mouse_press_coord, mouse_move_coord)
            except ZeroDivisionError:
                pass

        return selection_box

    @staticmethod
    def get_real_box(selected_box: Tuple[int, int, int, int], original_image_size: Tuple[int, int],
                     displayed_image_size: Tuple[int, int]) -> Tuple[int, int, int, int]:
        return (int(selected_box[0] * original_image_size[0]/displayed_image_size[0]),
                int(selected_box[1] * original_image_size[1] /
                    displayed_image_size[1]),
                int(selected_box[2] * original_image_size[0] /
                    displayed_image_size[0]),
                int(selected_box[3] * original_image_size[1]/displayed_image_size[1]))



import tkinter as tk
import types
from tkinter import Tk, Frame, Canvas, Event, Menu, messagebox, filedialog, Toplevel
from typing import Tuple, Any
from PIL.ImageTk import PhotoImage
import inbac


class View():
    def __init__(self, master: Tk, initial_window_size: Tuple[int, int]):
        self.master: Tk = master
        self.frame: Frame = tk.Frame(self.master, relief=tk.FLAT)
        self.frame.pack(fill=tk.BOTH, expand=tk.YES)
        self.image_canvas: Canvas = Canvas(self.frame, highlightthickness=0)
        self.image_canvas.pack(fill=tk.BOTH, expand=tk.YES)
        self.master.geometry(
            str(initial_window_size[0]) + "x" + str(initial_window_size[1]))
        self.master.update()
        self.controller = None

        self.bind_events()
        self.create_menu()

    def bind_events(self):
        self.master.bind('z', self.save_next)
        self.master.bind('x', self.save)
        self.master.bind('<Left>', self.previous_image)
        self.master.bind('<Right>', self.next_image)
        self.master.bind('<ButtonPress-3>', self.next_image)
        self.master.bind('<ButtonPress-2>', self.previous_image)
        self.image_canvas.bind('<ButtonPress-1>', self.on_mouse_down)
        self.image_canvas.bind('<B1-Motion>', self.on_mouse_drag)
        self.image_canvas.bind('<ButtonRelease-1>', self.on_mouse_up)

        self.master.bind('<KeyPress-Shift_L>', self.enable_selection_mode)
        self.master.bind('<KeyPress-Control_L>', self.enable_selection_mode)
        self.master.bind('<KeyRelease-Shift_L>', self.disable_selection_mode)
        self.master.bind('<KeyRelease-Control_L>', self.disable_selection_mode)

        self.image_canvas.bind('<Configure>', self.on_resize)

    def create_menu(self):
        self.menu: Menu = Menu(self.master, relief=tk.FLAT)
        self.menu.add_command(label="Open", command=self.open_dialog)
        self.menu.add_command(
            label="Settings", command=self.create_settings_window)
        self.menu.add_command(label="About", command=self.show_about_dialog)
        self.menu.add_separator()
        self.menu.add_command(label="Exit", command=self.master.quit)
        self.master.config(menu=self.menu)

    def ask_directory(self) -> str:
        return filedialog.askdirectory(parent=self.master)

    def open_dialog(self):
        self.controller.select_images_folder()
        self.controller.load_images()

    def create_settings_window(self):
        settings_window = tk.Toplevel(self.master)
        settings_window.title("Settings")
        settings_window.geometry("{}x{}".format(400, 400))
        settings = types.SimpleNamespace()

        settings.aspect_ratio_checked = tk.IntVar()
        settings.aspect_ratio_checked.set(
            self.controller.model.args.aspect_ratio is not None)
        aspect_ratio_checkbox = tk.Checkbutton(
            settings_window, variable=settings.aspect_ratio_checked, onvalue=1, offvalue=0, text="Aspect Ratio")
        aspect_ratio_checkbox.grid(row=0, column=0, columnspan=2, sticky=tk.W)

        settings.aspect_ratio_x = tk.StringVar()
        if self.controller.model.args.aspect_ratio is not None:
            settings.aspect_ratio_x.set(
                str(self.controller.model.args.aspect_ratio[0]))
        aspect_ratio_x_entry = tk.Entry(
            settings_window, width=5, textvariable=settings.aspect_ratio_x, bg="white")
        aspect_ratio_x_entry.grid(row=1, column=0)

        settings.aspect_ratio_y = tk.StringVar()
        if self.controller.model.args.aspect_ratio is not None:
            settings.aspect_ratio_y.set(
                str(self.controller.model.args.aspect_ratio[1]))
        aspect_ratio_y_entry = tk.Entry(
            settings_window, width=5, textvariable=settings.aspect_ratio_y, bg="white")
        aspect_ratio_y_entry.grid(row=1, column=1)

        settings.resize_checked = tk.IntVar()
        settings.resize_checked.set(
            self.controller.model.args.resize is not None)
        resize_checkbox = tk.Checkbutton(
            settings_window, variable=settings.resize_checked, onvalue=1, offvalue=0, text="Resize")
        resize_checkbox.grid(row=2, column=0, columnspan=2, sticky=tk.W)

        settings.resize_x = tk.StringVar()
        if self.controller.model.args.resize is not None:
            settings.resize_x.set(str(self.controller.model.args.resize[0]))
        resize_x_entry = tk.Entry(
            settings_window, width=5, textvariable=settings.resize_x, bg="white")
        resize_x_entry.grid(row=3, column=0)

        settings.resize_y = tk.StringVar()
        if self.controller.model.args.resize is not None:
            settings.resize_y.set(str(self.controller.model.args.resize[1]))
        resize_y_entry = tk.Entry(
            settings_window, width=5, textvariable=settings.resize_y, bg="white")
        resize_y_entry.grid(row=3, column=1)

        save_button = tk.Button(settings_window, text="Save",
                                command=lambda: self.save_settings(settings_window, settings))
        save_button.grid(row=4, column=0)

        cancel_button = tk.Button(settings_window, text="Cancel",
                                  command=lambda: self.cancel_settings(settings_window))
        cancel_button.grid(row=4, column=1)

    def save_settings(self, settings_window: Toplevel, settings: types.SimpleNamespace):
        if settings.aspect_ratio_checked.get():
            self.controller.model.args.aspect_ratio = (
                int(settings.aspect_ratio_x.get()), int(settings.aspect_ratio_y.get()))
        else:
            self.controller.model.args.aspect_ratio = None
        if settings.resize_checked.get():
            self.controller.model.args.resize = (
                int(settings.resize_x.get()), int(settings.resize_y.get()))
        else:
            self.controller.model.args.resize = None
        settings_window.destroy()

    def cancel_settings(self, settings_window: Toplevel):
        settings_window.destroy()

    def show_about_dialog(self):
        messagebox.showinfo("About", "inbac " +
                            inbac.__version__, parent=self.master)

    def show_error(self, title: str, message: str):
        messagebox.showerror(title, message, parent=self.master)

    def display_image(self, image: PhotoImage) -> Any:
        return self.image_canvas.create_image(0, 0, anchor=tk.NW, image=image)

    def remove_from_canvas(self, obj: Any):
        self.image_canvas.delete(obj)

    def create_rectangle(self, box: Tuple[int, int, int, int], outline_color: str) -> Any:
        return self.image_canvas.create_rectangle(box, outline=outline_color)

    def change_canvas_object_coords(self, obj: Any, coords: Tuple[int, int]):
        self.image_canvas.coords(obj, coords)

    def get_canvas_object_coords(self, obj: Any) -> Any:
        return self.image_canvas.coords(obj)

    def move_canvas_object_by_offset(self, obj: Any, offset_x: int, offset_y: int):
        self.image_canvas.move(obj, offset_x, offset_y)

    def enable_selection_mode(self, event: Event = None):
        self.controller.model.enabled_selection_mode = True

    def disable_selection_mode(self, event: Event = None):
        self.controller.model.enabled_selection_mode = False

    def on_mouse_down(self, event: Event):
        self.controller.start_selection((event.x, event.y))

    def on_mouse_drag(self, event: Event):
        self.controller.move_selection((event.x, event.y))

    def on_mouse_up(self, event: Event):
        self.controller.stop_selection()

    def next_image(self, event: Event = None):
        self.controller.next_image()

    def previous_image(self, event: Event = None):
        self.controller.previous_image()

    def on_resize(self, event: Event = None):
        self.controller.display_image_on_canvas(
            self.controller.model.current_image)

    def save_next(self, event: Event = None):
        self.controller.save_next()

    def save(self, event: Event = None):
        self.controller.save()

    def set_title(self, title: str):
        self.master.title(title)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['inbac']

package_data = \
{'': ['*']}

install_requires = \
['pillow>=7.0,<8.0']

entry_points = \
{'console_scripts': ['inbac = inbac.inbac:main',
                     'test = tests.test_inbac:main']}

setup_kwargs = {
    'name': 'inbac',
    'version': '2.0.0',
    'description': 'Interactive batch cropper made for quick cropping of images.',
    'long_description': '# inbac\n\n![](https://raw.githubusercontent.com/weclaw1/inbac/master/resources/demo.gif)\n\n## Description\n\ninbac is an **in**teractive **ba**tch **c**ropper made for quick cropping of images.  \nI made this program because cropping thousands of images with image viewers or image manipulation programs was too slow.\n\n## Installation\n\nThere are three ways to install this application.\nThe easiest option is to download the single file executable created with PyInstaller, which can be found on the releases page.\nIf you have python installed you can install with pip by typing\n\n`pip install inbac`\nIf you want to use the current master branch you can clone the repository and run the project using poetry.\n\n## Development Requirements\n- [poetry](https://github.com/sdispater/poetry)\n- tkinter\n\nAfter installing above dependencies run `poetry install` in project directory to install remaining dependencies.\n\n## Examples\n\n`poetry run inbac /home/user/pictures/`  \nOpens images in /home/user/pictures/ and saves cropped images to /home/user/pictures/crops\n\n`poetry run inbac -a 1 1 -r 256 256 /home/user/pictures/ /home/user/crops/`  \nOpens images in /home/user/pictures/ in 1:1 ratio selection mode and saves images resized to 256x256px in /home/user/crops/ \n\n## Usage\n\n```\nusage: inbac [-h] [-a ASPECT_RATIO ASPECT_RATIO] [-r RESIZE RESIZE]\n             [-s SELECTION_BOX_COLOR] [-w WINDOW_SIZE WINDOW_SIZE]\n             [-f IMAGE_FORMAT] [-q IMAGE_QUALITY]\n             [input_dir] [output_dir]\n\ninbac - interactive batch cropper\n\nLeft Mouse Button                 - select part of image\n\nZ                                 - save selection and go to the next picture\n\nX                                 - save selection and stay on the same picture\n\nHold Left Shift or Left Ctrl      - drag selection\n\nRight Arrow or Right Mouse Button - go to next picture\n\nLeft Arrow or Middle Mouse Button - go to previous picture\n\npositional arguments:\n  input_dir             input directory (defaults to current working\n                        directory)\n  output_dir            output directory (defaults to folder crops in input\n                        directory)\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -a ASPECT_RATIO ASPECT_RATIO, --aspect_ratio ASPECT_RATIO ASPECT_RATIO\n                        selection should have specified aspect ratio\n  -r RESIZE RESIZE, --resize RESIZE RESIZE\n                        cropped image will be resized to specified width and\n                        height\n  -s SELECTION_BOX_COLOR, --selection_box_color SELECTION_BOX_COLOR\n                        color of the selection box (default is black)\n  -w WINDOW_SIZE WINDOW_SIZE, --window_size WINDOW_SIZE WINDOW_SIZE\n                        initial window size (default is 800x600)\n  -f IMAGE_FORMAT, --image_format IMAGE_FORMAT\n                        define the croped image format\n  -q IMAGE_QUALITY, --image_quality IMAGE_QUALITY\n                        define the croped image quality\n ```\n',
    'author': 'Robert W\xc4\x99c\xc5\x82awski',
    'author_email': 'r.weclawski@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/weclaw1/inbac',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import tkinter as tk
import os

from tkinter import filedialog, Tk
from argparse import Namespace

import inbac.parse_arguments as parse_args
from inbac.model import Model
from inbac.view import View
from inbac.controller import Controller


class Application():
    def __init__(self, args: Namespace, master: Tk):
        self.model: Model = Model(args)
        self.view: View = View(master, args.window_size)

        if args.input_dir is None:
            args.input_dir = filedialog.askdirectory(parent=master)
        args.output_dir = getattr(
            args, "output_dir", os.path.join(args.input_dir, "crops"))

        self.controller: Controller = Controller(self.model, self.view)

        if not os.path.exists(args.output_dir):
            self.controller.create_output_directory()

        self.view.controller = self.controller

    def run(self):
        self.view.master.mainloop()


def main():
    root = tk.Tk()
    root.title("inbac")
    app = Application(parse_args.parse_arguments(), master=root)

    app.run()


if __name__ == "__main__":
    main()
